import pygame
import abc as _abc


# Class represent wall game objects
class WallI(object):
    __metaclass__ = _abc.ABCMeta


# Class represent item game objects
class Item(object):
    __metaclass__ = _abc.ABCMeta


# Class represent item actor game objects
class IActor(object):
    __metaclass__ = _abc.ABCMeta


# Class represent player game objects
class Player(object):
    __metaclass__ = _abc.ABCMeta


# Class represent game objects to put in game
class Actor(pygame.sprite.Sprite):
    __metaclass__ = _abc.ABCMeta

    @_abc.abstractmethod
    def __init__(self, name):
        pygame.sprite.Sprite.__init__(self)
        self.name = name
        self._sprite = None
        self.rect = None
        self.world = None

    @_abc.abstractmethod
    def update(self):
        pass

    def get_groups(self):
        return self.groups()

    def set_group(self, group):
        self.add(group)

    def set_world(self, world):
        self.world = world

    def set_animation(self, spritesheet):
        self._sprite = spritesheet
        self.rect = spritesheet.image.get_rect()

    def get_image(self):
        return self._sprite

    @property
    def x(self):
        return self.rect.x

    @x.setter
    def x(self, value):
        self.rect.x = value

    @property
    def y(self):
        return self.rect.y

    @y.setter
    def y(self, value):
        self.rect.y = value

    def get_intersecting_actors(self):
        intersectingList = []
        x = self.x
        y = self.y
        w = self.rect.width
        h = self.rect.height

        for list in self.world.worldList:
            for sprite in list:
                sx = sprite.rect.x
                sy = sprite.rect.y
                sw = sprite.rect.width
                sh = sprite.rect.height
                if (not((x >= sx + sw) or (x + w <= sx) or (y >= sy + sh) or (y + h <= sy))):
                    intersectingList.append(sprite)
        return intersectingList

    def intersectsWithWall(self, dx, dy):
        x = self.x + dx
        y = self.y - dy
        w = self.rect.width
        h = self.rect.height

        for wall in self.world.walls:
            if ((x >= wall.x + wall.width) or
                    (x + w <= wall.x) or
                    (y >= wall.y + wall.height) or
                    (y + h <= wall.y)):
                continue
            else:
                return True


# Class represent game objects that write to screen
class WriteToDisplay(object):
    __metaclass__ = _abc.ABCMeta

    @_abc.abstractmethod
    def __init__(self):
        self.font = pygame.font.SysFont("monospace", 15)
        self.color = (255, 255, 255)

    def setFont(self, font, size):
        self.font = pygame.font.SysFont(font, size)

    def write(self, infotext, dest, world, antialias=1):
        info = self.font.render(infotext, antialias, self.color)
        world.screen.blit(info, dest)


# Pull individual sprites from sprite sheets
class SpriteSheet(object):
    """ Class used to grab images out of a sprite sheet. """

    def __init__(self, gameObject, file_name, sw, sh, frames):
        """ Constructor. Pass in the file name of the sprite sheet. """
        self.gameObject = gameObject
        self.black = (0, 0, 0)

        self.sprite_sheet = pygame.image.load(file_name).convert_alpha()
        self.images = self.get_all_images(self.sprite_sheet.get_rect().width,
                                          self.sprite_sheet.get_rect().height,
                                          sw, sh, frames)
        self.image = self.images[0]
        self.gameObject.image = self.image
        self.pingpong = True
        # self.iterator = iter(self.images)

        self.fps = 60
        self.anim_speed = 3
        self.play_frame = round(self.fps / (frames * self.anim_speed))
        self.anim_frames = frames - 1
        self.current_frame = 1
        self.current_anim_frame = 0
        self.plus = True

    def play(self):
        if(self.anim_frames > 0):
            if(self.current_frame == self.play_frame):
                if(self.pingpong is False):
                    if(self.current_anim_frame < self.anim_frames):
                        self.current_anim_frame += 1
                    else:
                        self.current_anim_frame = 0
                else:
                    if(self.plus is True and self.current_anim_frame < self.anim_frames):
                        self.current_anim_frame += 1
                        if(self.current_anim_frame == self.anim_frames):
                            self.plus = False
                    elif(self.plus is False and self.current_anim_frame > 0):
                        self.current_anim_frame -= 1
                        if (self.current_anim_frame == 0):
                            self.plus = True
            self.gameObject.image = self.images[self.current_anim_frame]

            if(self.current_frame < self.play_frame):
                self.current_frame += 1
            else:
                self.current_frame = 1

    def stop(self):
        pass

    def get_all_images(self, fwidth, fheight, swidth, sheight, frames):
        """ Return single images from larger spritesheet in list.
            fwidth/fheight is full with and height of the picture,
            swidth/sheight is single sprite width and height of
            the picture. Frames is number of frames in spritesheet"""
        images = []
        total = 0
        for i in range(0, fheight, sheight):
            for y in range(0, fwidth, swidth):
                images.append(self.get_image(y, i, swidth, sheight))
                total += 1
                if (total >= frames):
                    break
        return images

    def get_image(self, x, y, iwidth, iheight):
        """ Grab a single image out of a larger spritesheet
            Pass in the x, y location of the sprite
            and the width and height of the sprite. """

        # Create a new blank image
        image = pygame.Surface([iwidth, iheight]).convert()

        # Copy the sprite from the large sheet onto the smaller image
        image.blit(self.sprite_sheet, (0, 0), (x, y, iwidth, iheight))

        # Assuming black works as the transparent color
        image.set_colorkey(self.black)

        # Return the image
        return image


# ################################################ GAME WALL OBJECT ####################################################
class Wall(pygame.sprite.Sprite):
    def __init__(self, world, x, y, w, h, group):
        self.world = world
        self.groups = group     # Should be PyManMain.walls
        pygame.sprite.Sprite.__init__(self, self.groups)
        self.rect = pygame.Rect(x, y, w, h)
        self.x = x
        self.y = y
        self.width = w
        self.height = h
        self.rect.x = x
        self.rect.y = y
        self.image = pygame.Surface([self.width, self.height])
        color = 255, 255, 255, 255
        self.image.set_colorkey(color)
        self.image.fill(color)



import pygame

# Define some colors
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
RED = (255, 0, 0)
GREEN = (0, 255, 0)
BLUE = (0, 0, 255)
# Define map defaults
DEFAULT_MAP = 'Map0.tmx'


class PyManMain:
    """The Main PyMan Class - This class handles the main
    initialization and creating of the Game."""

    def __init__(self, fps=60, w_offset=100, h_offset=100):
        """Initialize PyGame"""
        pygame.init()
        """Set the window Size"""
        self.w_offset = w_offset
        self.h_offset = h_offset
        self._width = 1
        self._height = 1

        """Set game related properties"""
        pygame.display.set_caption('Game Controller Master Thesis')
        self.fps = fps
        self.paused = False
        self.gameExit = False
        self.player = False

        """User functions to be executed"""
        self.before_start = []
        self.first_game_loop = []
        self.last_game_loop = []
        self.on_exit = []

        """Initialize game groups"""
        self.userInstance = pygame.sprite.Group()
        self.playerList = pygame.sprite.Group()
        self.itemList = pygame.sprite.Group()
        self.actorList = pygame.sprite.Group()
        self.walls = pygame.sprite.Group()

        """Fill game groups"""
        self.worldList = []
        self.worldList.append(self.itemList)
        self.worldList.append(self.actorList)
        self.worldList.append(self.userInstance)
        self.worldList.append(self.playerList)
        self.worldList.append(self.walls)

        """Intialize properties"""
        self.screen = None
        self.background = None
        self.map = None
        self.map_img = None

    def _createWorld(self, map=None):
        from gamecontrollermt.mapLoader import TiledMap

        """Create and Fill the Screen"""
        self.screen = pygame.display.set_mode((self._width, self._height))

        """Setting Up Map"""
        if(map is None):
            self.map = TiledMap('data/Maps/' + DEFAULT_MAP, self, self._width, self._height)
        else:
            try:
                self.map = TiledMap('data/Maps/' + map + '.tmx', self, self._width, self._height)
            except Exception as ex:
                print(str(ex))
                self.map = TiledMap('data/Maps/' + DEFAULT_MAP, self, self._width, self._height)

        """Resize map accordingly to tiled map"""
        self.screen = pygame.display.set_mode((self.map.width + self.w_offset, self.map.height + self.h_offset))
        self._width = self.map.width + self.w_offset
        self._height = self.map.height + self.h_offset
        self.map.worldWidthZero = ((self._width) - self.map.width)/2
        self.map.worldHeightZero = ((self._height) - self.map.height)/2

        """Fill background"""
        self.background = pygame.Surface(self.screen.get_size())
        self.background = self.background.convert()
        self.background.fill(BLACK)

        """Create and Spawn Map"""
        self.map_img = self.map._make_map()
        self.map._fill_map()

    def _mainLoop(self):
        for function in self.before_start:
            function()

        """This is the Main Loop of the Game"""
        clock = pygame.time.Clock()

        while not self.gameExit:
            for function in self.first_game_loop:
                function()

            """Pygame Events"""
            for event in pygame.event.get():
                if (event.type == pygame.QUIT):
                    self.gameExit = True

            if(self.paused):
                continue
            """Set game Frame rate"""
            clock.tick(self.fps)

            """Map blitting"""
            self.screen.blit(self.background, (0, 0))
            self.screen.blit(self.map_img, (self.map.worldWidthZero, self.map.worldHeightZero))

            """Update Screen"""
            for group in self.worldList:
                group.update()
                group.draw(self.screen)

            pygame.display.flip()

            for function in self.last_game_loop:
                function()

        for function in self.on_exit:
            function()

        pygame.quit()

    def pause(self):
        self.paused = not self.paused

    def add_group(self, group):
        self.worldList.append(group)

    def add_actor_to_game(self, actor_name, x, y, width=0, height=0, object_properties=[]):
        self.map.add_actor_to_map(actor_name, x, y, width, height, object_properties)

    def add_to_group(self, actor, group):
        if (not (actor in group)):
            group.add(actor)

    def remove_from_group(self, actor, group):
        if(actor in group):
            group.remove(actor)

# if __name__ == "__main__":
#     MainWindow = PyManMain()
#     MainWindow._createWorld()
#     MainWindow._mainLoop()
